	REM MazezaM (pronounced may-zam)
	REM programmed by Aaron Williams 
	REM April 1st 2018 - (in progress)
	REM 
	REM The finished game will be entered into the 
	REM Intellivision IntyBASIC Programming Contest 2018
	REM
	REM MazezaM was invented by Malcolm Tyrrell in 2000.
	REM https://sites.google.com/site/malcolmsprojects/mazezam-home-page 
	REM The game is licensed under the GPL.
	REM
	REM The original MazezaM was written for the ZX Spectrum in 2002.
	REM This port contains the 30 levels found in his 2010 version of the game.
	REM The levels have at most 14 interior columns and 11 interior rows.
	REM It also contains levels from research papers (TODO add more info later).
	REM
	REM Version 0.1 April 3rd, 2018
	REM   - Initial post on Atari Age INTYBASIC Programming context forum.
	REM   - All 30 ZX Spectrum 1K levels by Malcolm Tyrrell are implemented.
	REM
	REM Version 0.2 June 14th, 2018 
	REM   - Added level names into the DATA.
	REM   - Added level intro screens.
	REM   - Added a win screen (untested).
	REM   - Cleaned up the main game loop using GOSUB instead of GOTO.
	REM   - First attempt at saying "MazezaM" using Intellivoice.
	REM
	REM Version 0.3 June 18th, 2018
	REM   - level selection screen (using color square mode) 
	REM   - levels are centered on screen
	REM   - pushing sprites 
	REM   - character is now named HannaH
	REM   - basic sound and Intellivoice (currently disabled)
	REM   - Gray code levels are added (currently disabled)
	REM
	REM TODO
	REM   - add levels for a new quest
	REM   - undo
	REM   - saving
	REM   - add optimal moves for each level
	REM   - password?
	REM   - "good" ending for perfect scores
	REM   - nice title screen (using SCREEN?)
	REM   - note: you could conceivably ditch having the 16-bit row variables
	REM
	REM PS hey, what about rush hour with only horizontally moving boxes?
	
	REM Naming Conventions
	REM Procedure labels are written like ProcedureLabel.
	REM Line labels are written like lineLabel.
	REM Constants are written like CONSTANT_NAME.
	REM Variables are written like variable_name.

	' TODO global vs local variable conventions

REM Initialization
REM --------------
REM Set up the color stack mode, constants, arrays, game mode, etc.
REM

	INCLUDE "constants.bas"

	WAIT
	DEFINE DEF00,12,bitmapData
	MODE SCREEN_CS, CS_BLACK, CS_WHITE, CS_TAN, CS_YELLOW
	WAIT

	CONST SKIP_INPUT_FRAMES	= $08 ' input buffering

	CONST CARD_EMPTY		= $00 ' TODO use different bits for flags
	CONST CARD_EXIT			= $01 
	CONST CARD_ENTER		= $02 
	CONST CARD_BOX			= $03
	CONST CARD_WALL			= $04
	CONST CARD_PLAYER		= $05 ' deprecate this one?
	CONST CARD_OUTSIDE		= $06

	CONST MODE_TITLE		= $01 
	CONST MODE_LEVEL_SELECT	= $02 
	CONST MODE_STORY		= $03 
	CONST MODE_LEVEL_INTRO	= $04 
	CONST MODE_LEVEL_PLAY	= $05 
	CONST MODE_LEVEL_OUTRO	= $06 

	CONST PLAYER_STAND		= $01
	CONST PLAYER_FACE_RIGHT	= $02
	CONST PLAYER_PUSH_RIGHT	= $03
	CONST PLAYER_FACE_LEFT	= $04
	CONST PLAYER_PUSH_LEFT	= $05

	CONST LEVEL_SELECT_COLS = $06
	CONST LEVEL_SELECT_ROWS = $05 ' these should multiply to NUM_LEVELS
	CONST LEVEL_SELECT_WIDE = $03 ' width in characters of each entry
	CONST LEVEL_SELECT_TALL = $01 ' height in characters of each entry
	CONST LEVEL_SELECT_TOP  = $07
	CONST LEVEL_SELECT_LEFT = $01

	CONST MAX_MZ_ROWS		= $0C ' TODO Level 18 has 11 MazezaM rows
	CONST NUM_LEVELS		= $1e ' TODO currently hiding Gray code levels

	'CONST DATA_LEVEL_SIZE   = $22 ' 34 16-bit values per level 
	'CONST DATA_LEVEL_START  =   ' TODO can you compute the starting data sddress?

	' the 16-bit variables we use are unsigned
	' this is because the level "Speaking Frankly 5" has 16 columns
	unsigned #mz_row_bits
	unsigned #mz_row_bits_array
	unsigned #color_square

	' level information
	' TODO do you want to support levels with more than 16 columns?
	' if yes, then you need more bits for the array below
	DIM #mz_row_bits_array(MAX_MZ_ROWS)
	level_num = 1

	' color information (this is a bit wasteful but makes somethings easier)
	DIM draw_row_colors(MAX_MZ_ROWS)

	' game information
	game_mode = MODE_TITLE
	
	' currently allow players to access all 
	levels_cleared = 29 ' TODO hack / save games

	gosub VoiceIntro
	

	GOTO gameLoop



REM Game Loop
REM ---------
REM The main game loop.  
REM  - All calls from here should be GOSUBs and not GOTOs.
REM  - All changes to the game_mode should be made here.
REM Note: I did not know about ON FRAME when initially writing this game.

gameLoop:
	if game_mode = MODE_TITLE then 
		GOSUB TitleScreen
		game_mode = MODE_LEVEL_SELECT
	elseif game_mode = MODE_LEVEL_SELECT then 
		GOSUB LevelSelectScreen
		game_mode = MODE_LEVEL_INTRO
		if level_num = 1 then game_mode = MODE_STORY
	elseif game_mode = MODE_STORY then 
		GOSUB StoryScreen
		game_mode = MODE_LEVEL_INTRO
	elseif game_mode = MODE_LEVEL_INTRO then 
		GOSUB PrepareLevel
		GOSUB LevelIntroScreen
		GOSUB ResetLevel
		game_mode = MODE_LEVEL_PLAY
	elseif game_mode = MODE_LEVEL_PLAY then  
		GOSUB PlayLevel
		level_num = level_num + 1 ' TODO support exit to level select screen
		game_mode = MODE_LEVEL_OUTRO
	elseif game_mode = MODE_LEVEL_OUTRO then 
		gosub DrawYay
		gosub MusicLevelOutro
		for temp = 1 to 60
			WAIT
		next temp
		if level_num > NUM_LEVELS then
			GOSUB WinScreen ' TODO add good ending
			game_mode = MODE_TITLE  ' TODO change to level select
		else
			game_mode = MODE_LEVEL_INTRO
		end if
	end if
	
	GOTO gameLoop


REM Individual Screens
REM ------------------
REM These are individual screens that have control of the program until exited.
REM
REM

TitleScreen:	Procedure
	gosub ClearScreen
	
	print at  0, "                    "
	
	print color CS_RED
	print at  40, "      MazezaM       "	
	print color CS_YELLOW
	print at  60, "    Version 0.3     "
	print color CS_GREEN
	print at  80, " INTYBASIC Contest 3"
	
	print color CS_BLUE
	'print at 120, "   ZX Spectrum 1K   "
	print at 120, "    Classic Quest   "
	print at 140, "      New Quest     "
	
	print color CS_TAN
	print at 200, "Malcolm Tyrell(2000)"
	print at 220, "Aaron Williams(2018)"

	skip_input = 0
	quest_num = 1 
	while CONT1.BUTTON = 0 or quest_num > 1
		' debounce input
		while skip_input > 0 
			skip_input = skip_input - 1
			WAIT
			WAIT
		wend

		' handle input controls
		if CONT1.LEFT or CONT1.RIGHT or CONT1.UP or CONT1.DOWN then
			if quest_num = 1 then quest_num = 2 else quest_num = 1
			skip_input = SKIP_INPUT_FRAMES
		end if

		' draw arrow
		if quest_num = 1 then
			sc_col = 3
			sc_row = 6
		elseif quest_num = 2 then
			sc_col = 5
			sc_row = 7
		end if
		SPRITE 0, (sc_col+1)*8+HIT+VISIBLE, (sc_row+1)*8+FLIPX, $3F*8+CS_WHITE 
	wend

	' clear input controls before proceeding
	do until CONT1.BUTTON = 0: loop
	WAIT
	END


' convert from level select to screen co-ordinates.
LsToSc:	Procedure
	ls_row = (level_num-1) / LEVEL_SELECT_COLS
	ls_col = (level_num-1) - (ls_row * LEVEL_SELECT_COLS)

	sc_col = LEVEL_SELECT_LEFT + ls_col * LEVEL_SELECT_WIDE
	sc_row = LEVEL_SELECT_TOP  + ls_row * LEVEL_SELECT_TALL
	sc_pos = 20*sc_row + sc_col
		
	END


LevelSelectScreen:	Procedure
	gosub ClearScreen

	'print at   0, "       LeveL        "
	
	for level_num = 1 to NUM_LEVELS
		if level_num <= levels_cleared+1 then print color CS_WHITE else print color CS_TAN
		gosub LsToSc
		'print at sc_pos, <.LEVEL_SELECT_WIDE>level_num
		print at sc_pos, <.3>level_num
	next level_num
	
	' initialize the selected level
	level_num = levels_cleared + 1
	if level_num > NUM_LEVELS then level_num = NUM_LEVELS

	' handle input
	skip_input = 0
	change = 1
	while CONT1.BUTTON = 0 or level_num > levels_cleared+1
		while skip_input > 0 
			skip_input = skip_input - 1
			WAIT
			WAIT
		wend

		' handle controls
		if CONT1.LEFT then
			level_num = level_num - 1
			if level_num < 1 then level_num = NUM_LEVELS
			change = 1
		elseif CONT1.RIGHT then
			level_num = level_num + 1
			if level_num > NUM_LEVELS then level_num = 1
			change = 1
		elseif CONT1.UP then
			if level_num > LEVEL_SELECT_COLS then
				level_num = level_num - LEVEL_SELECT_COLS
			else
				level_num = NUM_LEVELS - LEVEL_SELECT_COLS + level_num
			end if
			change = 1
		elseif CONT1.DOWN then
			level_num = level_num + LEVEL_SELECT_COLS
			if level_num > NUM_LEVELS then level_num = level_num - NUM_LEVELS
			change = 1
		end if

		' update the arrow and the mini map
		if change = 1 then
			skip_input = SKIP_INPUT_FRAMES

			' draw arrow
			gosub LsToSc
			if level_num < 10 then sc_col = sc_col + 1
			SPRITE 0, (sc_col+1)*8+HIT+VISIBLE, (sc_row+1)*8+FLIPX, $3F*8+CS_WHITE 

			' handle the level select mini map
			gosub LevelSelectMiniMap
			
			' restore and print the level name
			sc_pos = 0
			gosub DrawLevelName
			
			change = 0
		end if

	wend
 
	' make sure the button is cleared before continuing
	do until CONT1.BUTTON = 0: loop
	WAIT
	END


LevelSelectMiniMap:	Procedure

	' read level data to prepare mini map
	gosub ReadLevelHeader
	gosub ReadLevelRowBits
	gosub SetLevelColors
	
	' compute the offsets
	sc_col_offset = (BACKGROUND_COLUMNS - mz_num_cols/2)/2 
	if mz_num_rows < 10 then sc_row_offset = 2 else sc_row_offset = 1
	sc_pos = sc_row_offset*20 + sc_col_offset

	' draw mini level
	gosub EraseMiniMap
	gosub DrawMiniMap

	END


StoryScreen:	Procedure
	gosub ClearScreen
	
	print color CS_WHITE
	
	print at   0, "       Story        "
	
	print at  40, " Our hero HannaH is "
	print at  60, "    challenging     "
	print at  80, " the MazezaM mazes. "

	print at 120, "Complete every LeveL"
	print at 140, "   to become the    "
	print at 160, "  MadaM of MazezaM. "

	' TODO make this a procedure
	do until CONT1.BUTTON = 0: loop
	do until CONT1.BUTTON: loop
	do until CONT1.BUTTON = 0: loop
	WAIT
	END
	
LevelIntroScreen:	Procedure
	gosub ClearScreen

	print color CS_WHITE
	print at   0, "                    "
	print at  40, "      Level "
	print <>level_num

	sc_pos = 60
	gosub DrawLevelName
	
	do until CONT1.BUTTON = 0: loop
	do until CONT1.BUTTON: loop
	do until CONT1.BUTTON = 0: loop
	WAIT
	END


WinScreen:	Procedure
	gosub ClearScreen

	print at   0, "                    "
	print at  80, "  Congratulations!  "	
	print at 180, " Master every level "
	print at 200, "for an amazaming end"
	print at 220, " (available by V1.0)"
	do until CONT1.BUTTON = 0: loop
	do until CONT1.BUTTON: loop
	do until CONT1.BUTTON = 0: loop
	WAIT
	END


REM Mini Level Procedures
REM ---------------------	
REM These procedures draw a mini map of a level using color squares mode.
REM

REM DrawSetColorSquare is deprecated but could be useful in the future.
' Computes the correct color for color square mode.
' Pre-Condition: corner0, corner1, corner2, corner3 are set to values 0-7
' which refer to colors black, blue, ..., yellow, background.  In particular,
' the value 7 is not white but background, which is the top of the color stack.
' Post-Condition: #color_square is properly set
'DrawSetColorSquare:	PROCEDURE
'	#color_square = CS_COLOUR_SQUARES_ENABLE
'	#color_square = #color_square + corner0
'	#color_square = #color_square + corner1*8
'	#color_square = #color_square + corner2*64
'	if corner3 <= 3 then
'		#color_square = #color_square + corner3*256*2
'	else 
'		#color_square = #color_square + 4096*2
'		#color_square = #color_square + (corner3-4)*256*2
'	end if
'	END

EraseMiniMap:	PROCEDURE
	print at  20 color CS_BLACK, "                    "
	print at  40 color CS_BLACK, "                    "
	print at  60 color CS_BLACK, "                    "
	print at  80 color CS_BLACK, "                    "
	print at 100 color CS_BLACK, "                    "
	print at 120 color CS_BLACK, "                    "

	END

' A mini version of the level in color squares mode.
' Can I do it?? yes!
' Pre-Condition: sc_pos?
' TODO this currently only supports up to 11 rows and not 12 rows
DrawMiniMap:	PROCEDURE
	for mz_row = 0 to mz_num_rows+1 
	
		' special case: exit early when the level has 11 rows
		if mz_row = 12 then exit for

		' get the row color and row bits
		if mz_row = 0 or mz_row = mz_num_rows+1 then 
			row_color = CS_TAN
			#mz_row_bits = $FFFF
		else
			row_color = draw_row_colors(mz_row-1)
			#mz_row_bits = #mz_row_bits_array(mz_row-1)
		end if

		
		for mz_col = 0 to mz_num_cols+1

			' get the backtab index
			' TODO there is a screenpos routine
			backtab_index = sc_pos + 20*(mz_row/2) + mz_col/2
											
			' get background card and mask out CS_COLOUR_SQUARES_ENABLE = #1000 
			#color_square = #BACKTAB(backtab_index)
			#color_square = #color_square and $efff
		
			' set the corner color and increment bits (if necessary)
			if mz_col = 0 then
				if mz_row = mz_enter_row then
					#corner_color = CS_BLACK
				else
					#corner_color = CS_TAN
				end if
			elseif mz_col = mz_num_cols+1 then
				if mz_row = mz_exit_row then
					#corner_color = CS_BLACK
				else
					#corner_color = CS_TAN
				end if
			elseif #mz_row_bits and $8000 then 
				#corner_color = row_color
				#mz_row_bits = #mz_row_bits * 2
			else
				#corner_color = CS_BLACK
				#mz_row_bits = #mz_row_bits * 2
			end if

			' update the appropriate corner of the card
			' note: this assumes that the corner is currently black
			if (mz_row and $01) then
			 	if (mz_col and $01) then ' pix3
					if #corner_color <= 3 then 
						#color_square = #corner_color*512 + #color_square
					else 
						#color_square = 8192 + (#corner_color-4)*512 + #color_square
					end if
				else ' pix2
					#color_square = #corner_color*64 + #color_square
				end if
			else
				if (mz_col and $01) then ' pix1
					#color_square = #corner_color*8 + #color_square
				else ' pix0
					#color_square =  #corner_color + #color_square
				end if			
			end if 

			' print the updated card with CS_COLOUR_SQUARES_ENABLE
			print at backtab_index, #color_square + CS_COLOUR_SQUARES_ENABLE

		next mz_col
		
	next mz_row
	
	
	
	' TODO hack
	print color CS_WHITE
	
	'
	'sc_pos = sc_pos - mz_num_cols/2 + 20
	
	' draw next two rows
	'#mz_row_bits_a = #mz_row_bits_array(1)		
	'#mz_row_bits_b = #mz_row_bits_array(2)		
	'for mz_col=0 to mz_num_cols-1 step 2
	'	if #mz_row_bits_a and $8000 then corner0 = draw_row_colors(1) else corner0 = CS_BLACK
	'	#mz_row_bits_a = #mz_row_bits_a * 2
	'	if #mz_row_bits_a and $8000 then corner1 = draw_row_colors(1) else corner1 = CS_BLACK
	'	if #mz_row_bits_b and $8000 then corner2 = draw_row_colors(2) else corner2 = CS_BLACK
	'	#mz_row_bits_b = #mz_row_bits_b * 2
	'	if #mz_row_bits_b and $8000 then corner3 = draw_row_colors(2) else corner3 = CS_BLACK
	'	
	'	gosub DrawSetColorSquare
	'	print at sc_pos + mz_col/2 + 20, #color_square
	'next mz_col
	
	'WAIT
	END



REM Level Procedures
REM ----------------
REM These procedures handle the playing of an individual level.
REM A level needs to be prepared once, then reset once before playing it.
REM Some of these procedures are used by the Level Select Mini Map.

' note: this doesn't read in the level bits ... that is done in reset level
PrepareLevel:	Procedure

	gosub ReadLevelHeader
	
	' compute the screen offsets
	sc_row_offset = (BACKGROUND_ROWS - 1 - mz_num_rows)/2 
	sc_col_offset = (BACKGROUND_COLUMNS - 2 - mz_num_cols)/2 

	'WAIT
	END


' note: this needs to be done after reading the level row bits'
SetLevelColors: PROCEDURE
	' initialize the view's row colors
	' we skip over CS_BLACK, CS_TAN (border), and CS_WHITE (color squares issue)
	' but CS_TAN is used for stationary rows
	color = CS_BLUE
	for mz_row=1 to mz_num_rows

		' set row color
		gosub MzIsStationaryRow
		if return_value > 0 then
			draw_row_colors(mz_row-1) = CS_TAN
		else
			draw_row_colors(mz_row-1) = color
			color = color + 1
			if color >= CS_WHITE then color = CS_BLUE
			if color = CS_TAN then color = color + 1
		end if
	next mz_row	

	END


ResetLevel:	Procedure

	gosub ReadLevelRowBits
	gosub SetLevelColors
		
	' initialize player location and pose in the mazezam level
	player_mz_row = mz_enter_row
	player_mz_col = 1
	player_pose = PLAYER_STAND

	'WAIT
	END


PlayLevel:	PROCEDURE

	gosub ClearScreen
	gosub DrawLevel

	while 1=1 ' TODO true doesn't seem to exist

		' draw the player
		gosub DrawPlayer

		' draw debug information
		'gosub DrawDebugInfo

		' check for level reset
		if CONT1.BUTTON then
			print at 0 color 7, "retry"
			for temp = 1 to 60
				WAIT
			next temp
			gosub ResetLevel
			gosub ClearScreen
			gosub DrawLevel
		end if

		' move the player
		gosub MzMovePlayer

		' check if the level is over and 
		' TODO this is a bit of a hack
		if player_mz_col = mz_num_cols+1 then 
			goto playLevelEnd
		end if
		
		' TODO better understand when to wait
		WAIT
	wend

playLevelEnd:
	'WAIT
	END




REM Drawing Procedures
REM ------------------
REM The procedures are involved in drawing.
REM

ClearScreen:	Procedure
	CLS
	
	' TODO loop over all sprites and erase them
	SPRITE 0,0 ',0,0
	SPRITE 1,0 ',0,0
	SPRITE 2,0 ',0,0
	SPRITE 3,0 ',0,0

	'WAIT
	END

' Convert MazezaM co-ordinates to background screen co-ordinates and position.
' More specifically, this function will center the MazezaM level on the screen.
'
' Pre-Conditions: 
'  - mz_row and mz_col give a valid position in the MazezaM level.
'  - mz_num_rows and mz_num_cols give the size of the MazezaM level,
'  - BACKGROUND_ROWS and BACKGROUND_COLUMNS give the size of the background.
'  - sc_row_offset and sc_col_offset have been computed (via PrepareLevel).
'
' Post-Condition: 
'  - sc_row and sc_col give the corresponding background co-ordinate.
'  - sc_pos give the corresponding background position.
MzToSc:	PROCEDURE
	sc_row = mz_row + sc_row_offset
	sc_col = mz_col + sc_col_offset
	sc_pos = 20*sc_row + sc_col
	'WAIT
	END


' When the player finishes a level this routine displays a Congratulatory message.
' The message should be a palindrome like 'yay' or 'wow'.
' TODO the message should be different when completed perfectly.
' TODO you could have give up messages like SOS and HUH and POOP lol
DrawYay:	PROCEDURE
	mz_row = 0
	mz_col = mz_num_cols - 2
	gosub MzToSc
	temp = RAND(4)
	if temp = 0 then
		print at sc_pos color 7, "YAY!"
	elseif temp = 1 then
		print at sc_pos color 7, "WOW!"
	elseif temp = 2 then
		print at sc_pos color 7, "AHA!"
	else
		print at sc_pos color 7, "HAH!"
	end if
	
	END


' draw level for the first time
DrawLevel:	PROCEDURE

	gosub ClearScreen

	' draw top row of walls but only if there are fewer than 12 rows.
	' (currently there are no rows with 12 rows)
	if mz_num_rows < 12 then
		print color CS_TAN
		mz_row = 0
		for mz_col=0 to mz_num_cols+1
			gosub MzToSc
			print at sc_pos, "\95"
		next mz_col
	end if

	' draw left wall but only if there are fewer than 19 columns
	if mz_num_cols < 19 then
		mz_col = 0
		for mz_row=1 to mz_num_rows
			if mz_row <> mz_enter_row then 
				gosub MzToSc
				print at sc_pos, "\95"
			end if
		next mz_row
	end if

	' draw right wall but only if there are fewer than 20 columns
	if mz_num_cols < 20 then
		mz_col = mz_num_cols+1
		for mz_row=1 to mz_num_rows
			if mz_row <> mz_exit_row then 
				gosub MzToSc
				print at sc_pos, "\95"
			end if
		next mz_row
	end if

	' draw bottom row of walls but only if there are fewer than 11 rows.
	' The levels Up Up And Away and Speaking Frankly 5 have 11 rows
	if mz_num_rows < 11 then
		mz_row = mz_num_rows+1
		for mz_col=0 to mz_num_cols+1
			gosub MzToSc
			print at sc_pos, "\95"
		next mz_col
	end if

	' draw the actual rows
	for mz_row=1 to mz_num_rows
		gosub DrawRow
	next mz_row

	'WAIT
	END


' Draw the boxes an individual row.  The borders are not drawn.
' Pre-Condition: mz_row has the MazezaM row to draw.
DrawRow:	PROCEDURE
	row_color = draw_row_colors(level_num)	
	#mz_row_bits = #mz_row_bits_array(mz_row-1)
	for mz_col=1 to mz_num_cols
		if #mz_row_bits and $8000 then
			row_color = draw_row_colors(mz_row-1)
			print color row_color
		else
			print color CS_BLACK
		end if
		gosub MzToSc
		print at sc_pos, "\95" 
		#mz_row_bits = #mz_row_bits * 2
	next mz_col
	WAIT
	END
	

' draw player
DrawPlayer:	PROCEDURE
	' do the intellivision sprites use the bottom-right position?
	mz_row = player_mz_row+1
	mz_col = player_mz_col+1
	gosub MzToSc
	
	player_sc_x = sc_col*8 ' this fits in 8-bits sincd 21*8 < 256
	player_sc_y = sc_row*8

	if player_pose = PLAYER_STAND then 
		SPRITE 0, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR00+SPR_RED
		SPRITE 1, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR01+SPR_TAN
		SPRITE 2, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR02+SPR_BLUE
		SPRITE 3, player_sc_x+HIT+VISIBLE, player_sc_y+4, SPR03+SPR_GREEN
	elseif player_pose = PLAYER_FACE_RIGHT then 
		SPRITE 0, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR04+SPR_RED
		SPRITE 1, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR05+SPR_TAN
		SPRITE 2, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR06+SPR_BLUE
		SPRITE 3, player_sc_x+HIT+VISIBLE, player_sc_y+4, SPR07+SPR_GREEN
	elseif player_pose = PLAYER_PUSH_RIGHT then 
		SPRITE 0, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR08+SPR_RED
		SPRITE 1, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR09+SPR_TAN
		SPRITE 2, player_sc_x+HIT+VISIBLE, player_sc_y,   SPR10+SPR_BLUE
		SPRITE 3, player_sc_x+HIT+VISIBLE, player_sc_y+4, SPR11+SPR_GREEN
	elseif player_pose = PLAYER_FACE_LEFT then 
		SPRITE 0, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX,   SPR04+SPR_RED
		SPRITE 1, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX,   SPR05+SPR_TAN
		SPRITE 2, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX,   SPR06+SPR_BLUE
		SPRITE 3, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX+4, SPR07+SPR_GREEN
	elseif player_pose = PLAYER_PUSH_LEFT then 
		SPRITE 0, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX,   SPR08+SPR_RED
		SPRITE 1, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX,   SPR09+SPR_TAN
		SPRITE 2, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX,   SPR10+SPR_BLUE
		SPRITE 3, player_sc_x+HIT+VISIBLE, player_sc_y+FLIPX+4, SPR11+SPR_GREEN
	end if

	'WAIT
	END


' sc_pos
DrawLevelName:	PROCEDURE
	gosub RestoreLevelName
	for temp=0 to BACKGROUND_COLUMNS-1
		read GROM_index
		print at sc_pos+temp color CS_WHITE, GROM_index*8+7
	next temp		
	END	


DrawDebugInfo: PROCEDURE
	' print out the most recent mz_card from MzGetCard
	print at 180, (mz_card+16)*8+7
	
	' print out the current player mz row and mz column
	print at 200 color 7, (player_mz_row+16)*8+7
	print at 201 color 7, (player_mz_col+16)*8+7
	
	' print out the most recent mz_row and mz_col likely from MzGetCard
	print at 220 color 7, (mz_row+16)*8+7
	print at 221 color 7, (mz_col+16)*8+7

	'WAIT
	END


REM Utilities
REM ---------
REM These procedures could be useful in INTYBasic programs other than MazezaM.
REM


REM Screen Co-ordinates
REM -------------------
REM We use two different co-ordinate systems.
REM  2. Screen co-ordinates are with respect to the 12-by-20 character background.
REM     For example, sc_row = sc_col = 0 refers to the top-left screen character.
REM We need use the first 
REM 


REM MazezaM Co-ordinates
REM --------------------
REM We use two different co-ordinate systems.
REM  1. MazezaM co-ordinates are with respect to the MazezaM level.
REM     For example, mz_row = mz_col = 0 refers to the top-left wall of the maze.
REM We need use the first 
REM 



REM New MazezaM Model
REM -----------------
REM todo
REM 

' Sets mz_card to be the card in position (mz_row, mz_col) of the MazezaM level.
' Pre-Condition: mz_row and mz_col are valid rows and columns for the level.
' Post-Condition: mz_card is set to the card type.
' TODO ohhh now I understand ... the top wall would be -1 and the left wall would be -1
' so we would need to use 1-indexing for inside of the maze ... hmmm, that is a big change
' let's write this function as if that change will occur
MzGetCard: PROCEDURE
	mz_card = CARD_OUTSIDE
	if mz_row > mz_num_rows+1 then goto mzGetCardEnd
	if mz_col > mz_num_cols+1 then goto mzGetCardEnd

	mz_card = CARD_PLAYER 
	if mz_row = player_mz_row AND mz_col = player_mz_col then goto mzGetCardEnd

	mz_card = CARD_WALL
	if mz_row = 0 then
		goto mzGetCardEnd
	elseif mz_row = (mz_num_rows+1) then
		goto mzGetCardEnd
	elseif mz_col = 0 then
		if mz_row = mz_enter_row then mz_card = CARD_ENTER
		goto mzGetCardEnd
	elseif mz_col = (mz_num_cols+1) then
		if mz_row = mz_exit_row then mz_card = CARD_EXIT
		goto mzGetCardEnd
	else
		mz_card = CARD_EMPTY
		#mz_row_bits = #mz_row_bits_array(mz_row-1)
		temp_mz_col = mz_col-1
		while temp_mz_col > 0 
			#mz_row_bits = #mz_row_bits * 2
			temp_mz_col = temp_mz_col - 1
		wend
		if #mz_row_bits and $8000 then mz_card = CARD_BOX
		goto mzGetCardEnd
	end if

mzGetCardEnd:
	'WAIT
	END




' Pre-Condition: (player_mz_row, player_mz_col) are valid positions in the MazezaM level.
' Post-Condition: If skip_input > 0, then skip_input is decremented and nothing else is done.
' Otherwise, the player is moved according to the controller.
MzMovePlayer:	PROCEDURE

	if skip_input > 0 then
		skip_input = skip_input - 1
		GOTO mzMovePlayerEnd
	end if

	' Attempt to move the player in the requested direction
	if CONT1.UP then
		mz_row = player_mz_row - 1
		mz_col = player_mz_col
		gosub MzGetCard
		if mz_card <> CARD_EMPTY then GOTO mzMovePlayerEnd
		player_mz_row = player_mz_row - 1
		player_pose = PLAYER_STAND
		skip_input = SKIP_INPUT_FRAMES
	elseif CONT1.DOWN then
		mz_row = player_mz_row + 1
		mz_col = player_mz_col
		gosub MzGetCard
		if mz_card <> CARD_EMPTY then GOTO mzMovePlayerEnd
		player_mz_row = player_mz_row + 1
		player_pose = PLAYER_STAND
		skip_input = SKIP_INPUT_FRAMES
	elseif CONT1.LEFT then 
		mz_row = player_mz_row
		mz_col = player_mz_col - 1
		gosub MzGetCard
		player_pose = PLAYER_FACE_LEFT
		if mz_card = CARD_WALL or mz_card = CARD_ENTER then 
			GOTO mzMovePlayerEnd
		elseif mz_card = CARD_EMPTY then
			player_mz_col = player_mz_col - 1
			skip_input = SKIP_INPUT_FRAMES
			GOTO mzMovePlayerEnd
		elseif mz_card = CARD_BOX then
			' check if this row of boxes can move
			mz_row = player_mz_row
			mz_col = 1
			gosub MzGetCard
			if mz_card = CARD_EMPTY then
				player_mz_col = player_mz_col - 1
				player_pose = PLAYER_PUSH_LEFT
				skip_input = SKIP_INPUT_FRAMES		
				#mz_row_bits_array(player_mz_row-1) = #mz_row_bits_array(player_mz_row-1) * 2
				mz_row = player_mz_row
				gosub DrawRow
			end if
			GOTO mzMovePlayerEnd
		end if
	elseif CONT1.RIGHT then
		mz_row = player_mz_row
		mz_col = player_mz_col + 1
		gosub MzGetCard
		player_pose = PLAYER_FACE_RIGHT
		if mz_card = CARD_WALL then 
			GOTO mzMovePlayerEnd
		elseif mz_card = CARD_EMPTY or mz_card = CARD_EXIT then
			player_mz_col = player_mz_col + 1
			skip_input = SKIP_INPUT_FRAMES
			GOTO mzMovePlayerEnd
		elseif mz_card = CARD_BOX then 
			' check if this row of boxes can move
			mz_row = player_mz_row
			mz_col = mz_num_cols
			gosub MzGetCard
			if mz_card = CARD_EMPTY then
				player_mz_col = player_mz_col + 1
				player_pose = PLAYER_PUSH_RIGHT
				skip_input = SKIP_INPUT_FRAMES
				#mz_row_bits_array(player_mz_row-1) = #mz_row_bits_array(player_mz_row-1)/2
				mz_row = player_mz_row
				gosub DrawRow
			end if
			GOTO mzMovePlayerEnd
		end if
	end if


mzMovePlayerEnd:
	' WAIT
	END

' Tests if a row is stationary.  A row can be stationary in three ways.
'  (1) There is a box on the far left and far right.
'  (2) The boxes form a prefix of the row.
'  (3) The boxes form a prefix of the row.
'
' Pre-Condition: 
'  - mz_row
'  - level_num
'  - #mz_row_bits_array
' Post-Condition: 
'  - return_value = 0 if the row is not stationary
MzIsStationaryRow:	Procedure	
	' default return value is 0 for not stationary
	return_value = 0
	
	' check for type (1)
	mz_col = 1
	gosub MzGetCard
	if mz_card = CARD_BOX then
		mz_col = mz_num_cols
		gosub MzGetCard
		if mz_card = CARD_BOX then 
			return_value = 1
			goto mzIsStationaryRowEnd
		end if
	end if
	
	' check for type (2)
	' in other words, check if the row is boxes followed by empties
	mz_col = 1
	gosub MzGetCard
	while mz_card = CARD_BOX
		mz_col = mz_col + 1
		if mz_col > mz_num_cols then
			return_value = 2
			goto mzIsStationaryRowEnd
		end if
		gosub MzGetCard
	wend
	while mz_card = CARD_EMPTY
		mz_col = mz_col + 1
		if mz_col > mz_num_cols then
			return_value = 2
			goto mzIsStationaryRowEnd
		end if
		gosub MzGetCard
	wend

	' check for type (3)
	' in other words, check if the row is boxes followed by empties
	mz_col = 1
	gosub MzGetCard
	while mz_card = CARD_EMPTY
		mz_col = mz_col + 1
		if mz_col > mz_num_cols then
			return_value = 3
			goto mzIsStationaryRowEnd
		end if
		gosub MzGetCard
	wend
	while mz_card = CARD_BOX
		mz_col = mz_col + 1
		if mz_col > mz_num_cols then
			return_value = 3
			goto mzIsStationaryRowEnd
		end if
		gosub MzGetCard
	wend

mzIsStationaryRowEnd:
	'WAIT
	END


	
REM Voice and Music
REM ---------------
REM Area of code for storing Intellivoice and Music information.
REM
REM Note: 
REM I am currently very suspicious of the Intellivoice.  
REM When I started using the VOICE commands by Liz the program became fragile.
REM More specifically, the program would immediately crash "sometimes".
REM When I changed the program in some seemingly trivial and inconsequential way
REM then suddenly the program would no longer crash.
REM When I stopped using the Intellivoice this stopped happening.
REM

VoiceIntro:	PROCEDURE
	'VOICE INIT
	'VOICE PLAY WAIT voiceMazezaM
	'WAIT
	END
	
'voiceMazezaM: VOICE MM, EH, EH, EH, ZZ, ZZ, HH1, AA, AA, AA, MM, PA4, ZERO, THREE, PA4, 0
'voiceMazezaM: VOICE MM, EH, EH, EH, AY, PA1, ZZ, ZZ, AA, MM, MM, PA4, 0

MusicLevelOutro:	PROCEDURE
	PLAY SIMPLE
	PLAY musicVictory
	WAIT
	END

musicVictory: ' taken from INTYBASIC sample lander.bas
	DATA 5
	MUSIC F4W,-,-,-
	MUSIC F4W,-,-,-
	MUSIC F4W,-,-,-
	MUSIC S,-,-,-
	MUSIC S,-,-,-
	MUSIC S,-,-,-
	MUSIC STOP


REM Restore and Read Data
REM ---------------------
REM These procedures restore data pointers and then read level data.  
REM The data was converted from levels.mzm using utils/mzm2intv.py.
REM Each level has the following parts which can be restored or read:
REM
REM  - The Name consisting of 20 characters (unpacked) that is space centered.
REM    This can later be optimized by either packing and/or removing spaces.
REM
REM  - The Header consisting of four bytes.
REM    1 byte: number of MazezaM rows (max: 12)
REM    1 byte: number of MazezaM columns (max: 20)
REM    1 byte: enter row (1-based indexing)
REM    1 byte: exit row (1-based indexing)
REM    This could be compressed into two bytes since 12*20 < 256.
REM
REM  - The Row Bits consist of the on/off pattern in the rows in hex.
REM
REM Note: The levels do not have the same size, so the restore commands are ugly.
REM Note: We do not store the name in any variable, so there is no read function.
REM
REM The following example is for Level 2 (Humble Origins).
REM
REM Name:
REM   This uses ASCII -> GROM and should be self-explanatory.
REM
REM Header:
REM   2 MazezaM rows
REM   7 MazezaM columns
REM   entrance row 0
REM   exit row 1
REM
REM   This gives the following header.
REM
REM   levelHeader02:
REM 	  DATA $0002,$0007
REM 	  DATA $0000,$0001
REM
REM Row Bits: 
REM   The actual level looks like the following.
REM   #########
REM   + $  $$ #
REM   # $  $  *
REM   #########
REM
REM   Converting this to bytes gives the following.
REM    $   $$ 
REM   0100 1100
REM   4    C
REM  
REM    $   $
REM   0100 1000
REM   4    8
REM
REM  Therefore, the level is completed with the last row below.
REM
REM  levelRowBits02:
REM      DATA $4c00,$4800


' set location for the level header DATA reads
RestoreLevelHeader: PROCEDURE
	if level_num=1  then restore levelHeader01
	if level_num=2  then restore levelHeader02
	if level_num=3  then restore levelHeader03
	if level_num=4  then restore levelHeader04
	if level_num=5  then restore levelHeader05
	if level_num=6  then restore levelHeader06
	if level_num=7  then restore levelHeader07
	if level_num=8  then restore levelHeader08
	if level_num=9  then restore levelHeader09
	if level_num=10 then restore levelHeader10
	if level_num=11 then restore levelHeader11
	if level_num=12 then restore levelHeader12
	if level_num=13 then restore levelHeader13
	if level_num=14 then restore levelHeader14
	if level_num=15 then restore levelHeader15
	if level_num=16 then restore levelHeader16
	if level_num=17 then restore levelHeader17
	if level_num=18 then restore levelHeader18
	if level_num=19 then restore levelHeader19
	if level_num=20 then restore levelHeader20
	if level_num=21 then restore levelHeader21
	if level_num=22 then restore levelHeader22
	if level_num=23 then restore levelHeader23
	if level_num=24 then restore levelHeader24
	if level_num=25 then restore levelHeader25
	if level_num=26 then restore levelHeader26
	if level_num=27 then restore levelHeader27
	if level_num=28 then restore levelHeader28
	if level_num=29 then restore levelHeader29
	if level_num=30 then restore levelHeader30
	if level_num=31 then restore levelHeader31
	if level_num=32 then restore levelHeader32
	if level_num=33 then restore levelHeader33
	if level_num=34 then restore levelHeader34
	if level_num=35 then restore levelHeader35
	END


ReadLevelHeader: PROCEDURE
	gosub RestoreLevelHeader
	read mz_num_rows ' number of MazezaM rows
	read mz_num_cols ' number of MazezaM columns
	read mz_enter_row
	read mz_exit_row
	END


RestoreLevelRowBits: PROCEDURE
	if level_num=1  then restore levelRowBits01
	if level_num=2  then restore levelRowBits02
	if level_num=3  then restore levelRowBits03
	if level_num=4  then restore levelRowBits04
	if level_num=5  then restore levelRowBits05
	if level_num=6  then restore levelRowBits06
	if level_num=7  then restore levelRowBits07
	if level_num=8  then restore levelRowBits08
	if level_num=9  then restore levelRowBits09
	if level_num=10 then restore levelRowBits10
	if level_num=11 then restore levelRowBits11
	if level_num=12 then restore levelRowBits12
	if level_num=13 then restore levelRowBits13
	if level_num=14 then restore levelRowBits14
	if level_num=15 then restore levelRowBits15
	if level_num=16 then restore levelRowBits16
	if level_num=17 then restore levelRowBits17
	if level_num=18 then restore levelRowBits18
	if level_num=19 then restore levelRowBits19
	if level_num=20 then restore levelRowBits20
	if level_num=21 then restore levelRowBits21
	if level_num=22 then restore levelRowBits22
	if level_num=23 then restore levelRowBits23
	if level_num=24 then restore levelRowBits24
	if level_num=25 then restore levelRowBits25
	if level_num=26 then restore levelRowBits26
	if level_num=27 then restore levelRowBits27
	if level_num=28 then restore levelRowBits28
	if level_num=29 then restore levelRowBits29
	if level_num=30 then restore levelRowBits30
	if level_num=31 then restore levelRowBits31
	if level_num=32 then restore levelRowBits32
	if level_num=33 then restore levelRowBits33
	if level_num=34 then restore levelRowBits34
	if level_num=35 then restore levelRowBits35
	END

ReadLevelRowBits: PROCEDURE
	gosub RestoreLevelRowBits
	for mz_row=1 to mz_num_rows
		read #mz_row_bits_array(mz_row-1)
	next mz_row
	END

RestoreLevelName: PROCEDURE
	if level_num=1  then restore levelName01
	if level_num=2  then restore levelName02
	if level_num=3  then restore levelName03
	if level_num=4  then restore levelName04
	if level_num=5  then restore levelName05
	if level_num=6  then restore levelName06
	if level_num=7  then restore levelName07
	if level_num=8  then restore levelName08
	if level_num=9  then restore levelName09
	if level_num=10 then restore levelName10
	if level_num=11 then restore levelName11
	if level_num=12 then restore levelName12
	if level_num=13 then restore levelName13
	if level_num=14 then restore levelName14
	if level_num=15 then restore levelName15
	if level_num=16 then restore levelName16
	if level_num=17 then restore levelName17
	if level_num=18 then restore levelName18
	if level_num=19 then restore levelName19
	if level_num=20 then restore levelName20
	if level_num=21 then restore levelName21
	if level_num=22 then restore levelName22
	if level_num=23 then restore levelName23
	if level_num=24 then restore levelName24
	if level_num=25 then restore levelName25
	if level_num=26 then restore levelName26
	if level_num=27 then restore levelName27
	if level_num=28 then restore levelName28
	if level_num=29 then restore levelName29
	if level_num=30 then restore levelName30
	if level_num=31 then restore levelName31
	if level_num=32 then restore levelName32
	if level_num=33 then restore levelName33
	if level_num=34 then restore levelName34
	if level_num=35 then restore levelName35
	END



REM End of Game Loop
REM ----------------
REM This code should never be reached.

endGame:
	GOTO endGame


REM Level 01 "First Principles"
levelName01:
	DATA $0000,$0000,$0026,$0049,$0052,$0053,$0054,$0000,$0030,$0052,$0049,$004e,$0043,$0049,$0050,$004c,$0045,$0053,$0000,$0000
levelHeader01:
	DATA $0002,$0005
	DATA $0002,$0001
levelRowBits01:
	DATA $5000,$4000

REM Level 02 "Humble Origins"
levelName02:
	DATA $0000,$0000,$0000,$0028,$0055,$004d,$0042,$004c,$0045,$0000,$002f,$0052,$0049,$0047,$0049,$004e,$0053,$0000,$0000,$0000
levelHeader02:
	DATA $0002,$0007
	DATA $0001,$0002
levelRowBits02:
	DATA $4c00,$4800

REM Level 03 "Baby Steps"
levelName03:
	DATA $0000,$0000,$0000,$0000,$0000,$0022,$0041,$0042,$0059,$0000,$0033,$0054,$0045,$0050,$0053,$0000,$0000,$0000,$0000,$0000
levelHeader03:
	DATA $0003,$0007
	DATA $0001,$0002
levelRowBits03:
	DATA $6600,$5400,$6800

REM Level 04 "Easy Does It"
levelName04:
	DATA $0000,$0000,$0000,$0000,$0025,$0041,$0053,$0059,$0000,$0024,$004f,$0045,$0053,$0000,$0029,$0054,$0000,$0000,$0000,$0000
levelHeader04:
	DATA $0003,$0008
	DATA $0003,$0002
levelRowBits04:
	DATA $2700,$2a00,$5400

REM Level 05 "Little Rascal"
levelName05:
	DATA $0000,$0000,$0000,$002c,$0049,$0054,$0054,$004c,$0045,$0000,$0032,$0041,$0053,$0043,$0041,$004c,$0000,$0000,$0000,$0000
levelHeader05:
	DATA $0004,$0005
	DATA $0004,$0004
levelRowBits05:
	DATA $4000,$5800,$5000,$4800

REM Level 06 "Leap-frog"
levelName06:
	DATA $0000,$0000,$0000,$0000,$0000,$002c,$0045,$0041,$0050,$000d,$0046,$0052,$004f,$0047,$0000,$0000,$0000,$0000,$0000,$0000
levelHeader06:
	DATA $0004,$0007
	DATA $0001,$0004
levelRowBits06:
	DATA $5400,$b800,$5600,$4800

REM Level 07 "Shuttles"
levelName07:
	DATA $0000,$0000,$0000,$0000,$0000,$0000,$0033,$0048,$0055,$0054,$0054,$004c,$0045,$0053,$0000,$0000,$0000,$0000,$0000,$0000
levelHeader07:
	DATA $0005,$0007
	DATA $0001,$0005
levelRowBits07:
	DATA $3a00,$2800,$2e00,$aa00,$2400

REM Level 08 "The Nudge"
levelName08:
	DATA $0000,$0000,$0000,$0000,$0000,$0034,$0048,$0045,$0000,$002e,$0055,$0044,$0047,$0045,$0000,$0000,$0000,$0000,$0000,$0000
levelHeader08:
	DATA $0003,$0008
	DATA $0003,$0003
levelRowBits08:
	DATA $a000,$2b00,$6e00

REM Level 09 "Opening Gambit"
levelName09:
	DATA $0000,$0000,$0000,$002f,$0050,$0045,$004e,$0049,$004e,$0047,$0000,$0027,$0041,$004d,$0042,$0049,$0054,$0000,$0000,$0000
levelHeader09:
	DATA $0004,$0007
	DATA $0002,$0003
levelRowBits09:
	DATA $2800,$6c00,$5400,$b800

REM Level 10 "Loop-de-Loop"
levelName10:
	DATA $0000,$0000,$0000,$0000,$002c,$004f,$004f,$0050,$000d,$0044,$0045,$000d,$002c,$004f,$004f,$0050,$0000,$0000,$0000,$0000
levelHeader10:
	DATA $0004,$000e
	DATA $0002,$0004
levelRowBits10:
	DATA $7db0,$1998,$cb70,$1fe4

REM Level 11 "Nudge-nudge"
levelName11:
	DATA $0000,$0000,$0000,$0000,$002e,$0055,$0044,$0047,$0045,$000d,$004e,$0055,$0044,$0047,$0045,$0000,$0000,$0000,$0000,$0000
levelHeader11:
	DATA $0004,$0007
	DATA $0002,$0002
levelRowBits11:
	DATA $6c00,$5400,$5c00,$1800

REM Level 12 "To and Fro"
levelName12:
	DATA $0000,$0000,$0000,$0000,$0000,$0034,$004f,$0000,$0041,$004e,$0044,$0000,$0026,$0052,$004f,$0000,$0000,$0000,$0000,$0000
levelHeader12:
	DATA $0006,$000d
	DATA $0001,$0001
levelRowBits12:
	DATA $1f00,$be70,$5de0,$a7f0,$7460,$ab20

REM Level 13 "Ragged Rascal"
levelName13:
	DATA $0000,$0000,$0000,$0032,$0041,$0047,$0047,$0045,$0044,$0000,$0032,$0041,$0053,$0043,$0041,$004c,$0000,$0000,$0000,$0000
levelHeader13:
	DATA $0005,$0006
	DATA $0004,$0002
levelRowBits13:
	DATA $4000,$6400,$6800,$5c00,$2000

REM Level 14 "Hold Your Horses"
levelName14:
	DATA $0000,$0000,$0028,$004f,$004c,$0044,$0000,$0039,$004f,$0055,$0052,$0000,$0028,$004f,$0052,$0053,$0045,$0053,$0000,$0000
levelHeader14:
	DATA $0004,$0008
	DATA $0003,$0004
levelRowBits14:
	DATA $3800,$3700,$3a00,$5000

REM Level 15 "Knot Theory"
levelName15:
	DATA $0000,$0000,$0000,$0000,$002b,$004e,$004f,$0054,$0000,$0034,$0048,$0045,$004f,$0052,$0059,$0000,$0000,$0000,$0000,$0000
levelHeader15:
	DATA $0006,$0006
	DATA $0001,$0006
levelRowBits15:
	DATA $2800,$a800,$2800,$b000,$b000,$2c00

REM Level 16 "Rack and Pinion"
levelName16:
	DATA $0000,$0000,$0032,$0041,$0043,$004b,$0000,$0041,$004e,$0044,$0000,$0030,$0049,$004e,$0049,$004f,$004e,$0000,$0000,$0000
levelHeader16:
	DATA $0004,$000a
	DATA $0001,$0003
levelRowBits16:
	DATA $3500,$eb00,$6540,$dd00

REM Level 17 "Blue Meanie"
levelName17:
	DATA $0000,$0000,$0000,$0000,$0022,$004c,$0055,$0045,$0000,$002d,$0045,$0041,$004e,$0049,$0045,$0000,$0000,$0000,$0000,$0000
levelHeader17:
	DATA $0004,$0007
	DATA $0001,$0004
levelRowBits17:
	DATA $6800,$4a00,$b800,$2400

REM Level 18 "Up, Up and Away"
levelName18:
	DATA $0000,$0000,$0035,$0050,$000c,$0000,$0035,$0050,$0000,$0041,$004e,$0044,$0000,$0021,$0057,$0041,$0059,$0000,$0000,$0000
levelHeader18:
	DATA $000b,$0005
	DATA $000b,$0001
levelRowBits18:
	DATA $2000,$5800,$6000,$a000,$5000,$7000,$a000,$5000,$5000,$b000,$4000

REM Level 19 "Great Things"
levelName19:
	DATA $0000,$0000,$0000,$0000,$0027,$0052,$0045,$0041,$0054,$0000,$0034,$0048,$0049,$004e,$0047,$0053,$0000,$0000,$0000,$0000
levelHeader19:
	DATA $0004,$0007
	DATA $0003,$0004
levelRowBits19:
	DATA $5000,$6c00,$2a00,$4e00

REM Level 20 "Through and Through"
levelName20:
	DATA $0034,$0048,$0052,$004f,$0055,$0047,$0048,$0000,$0041,$004e,$0044,$0000,$0034,$0048,$0052,$004f,$0055,$0047,$0048,$0000
levelHeader20:
	DATA $0004,$000f
	DATA $0003,$0001
levelRowBits20:
	DATA $79e6,$5b58,$5bd8,$59e4

REM Level 21 "Just Enough"
levelName21:
	DATA $0000,$0000,$0000,$0000,$002a,$0055,$0053,$0054,$0000,$0025,$004e,$004f,$0055,$0047,$0048,$0000,$0000,$0000,$0000,$0000
levelHeader21:
	DATA $0005,$0008
	DATA $0003,$0003
levelRowBits21:
	DATA $ac00,$d600,$1500,$bc00,$1400

REM Level 22 "The Secret Staircase"
levelName22:
	DATA $0034,$0048,$0045,$0000,$0033,$0045,$0043,$0052,$0045,$0054,$0000,$0033,$0054,$0041,$0049,$0052,$0043,$0041,$0053,$0045
levelHeader22:
	DATA $0004,$000b
	DATA $0001,$0003
levelRowBits22:
	DATA $7600,$66a0,$9ec0,$b000

REM Level 23 "Clear the Way"
levelName23:
	DATA $0000,$0000,$0000,$0023,$004c,$0045,$0041,$0052,$0000,$0054,$0048,$0045,$0000,$0037,$0041,$0059,$0000,$0000,$0000,$0000
levelHeader23:
	DATA $0005,$0008
	DATA $0004,$0002
levelRowBits23:
	DATA $2700,$1900,$5d00,$5600,$4b00

REM Level 24 "A Mismatch in Heaven"
levelName24:
	DATA $0021,$0000,$002d,$0049,$0053,$004d,$0041,$0054,$0043,$0048,$0000,$0049,$004e,$0000,$0028,$0045,$0041,$0056,$0045,$004e
levelHeader24:
	DATA $0004,$0009
	DATA $0004,$0001
levelRowBits24:
	DATA $4400,$de00,$ec00,$5000

REM Level 25 "Double Cross"
levelName25:
	DATA $0000,$0000,$0000,$0000,$0024,$004f,$0055,$0042,$004c,$0045,$0000,$0023,$0052,$004f,$0053,$0053,$0000,$0000,$0000,$0000
levelHeader25:
	DATA $0007,$0009
	DATA $0007,$0003
levelRowBits25:
	DATA $4f00,$4b00,$5e80,$b200,$2380,$7e00,$2000

REM Level 26 "Zen Garden"
levelName26:
	DATA $0000,$0000,$0000,$0000,$0000,$003a,$0045,$004e,$0000,$0027,$0041,$0052,$0044,$0045,$004e,$0000,$0000,$0000,$0000,$0000
levelHeader26:
	DATA $0006,$0007
	DATA $0001,$0001
levelRowBits26:
	DATA $4c00,$5800,$f400,$1600,$bc00,$0000

REM Level 27 "Inside Out"
levelName27:
	DATA $0000,$0000,$0000,$0000,$0000,$0029,$004e,$0053,$0049,$0044,$0045,$0000,$002f,$0055,$0054,$0000,$0000,$0000,$0000,$0000
levelHeader27:
	DATA $000a,$000d
	DATA $0008,$0001
levelRowBits27:
	DATA $0010,$7fc8,$7030,$5fd0,$7730,$5130,$5fb0,$5470,$7ff0,$0000

REM Level 28 "Double Decker"
levelName28:
	DATA $0000,$0000,$0000,$0024,$004f,$0055,$0042,$004c,$0045,$0000,$0024,$0045,$0043,$004b,$0045,$0052,$0000,$0000,$0000,$0000
levelHeader28:
	DATA $0007,$0009
	DATA $0002,$0005
levelRowBits28:
	DATA $4c00,$6a80,$ac00,$7f80,$2f00,$b200,$5c00

REM Level 29 "Back to Front"
levelName29:
	DATA $0000,$0000,$0000,$0022,$0041,$0043,$004b,$0000,$0054,$004f,$0000,$0026,$0052,$004f,$004e,$0054,$0000,$0000,$0000,$0000
levelHeader29:
	DATA $000a,$000b
	DATA $0001,$000a
levelRowBits29:
	DATA $0000,$ffc0,$ef00,$ad00,$0bc0,$fb00,$76c0,$0ce0,$7fe0,$0000

REM Level 30 "The Beast"
levelName30:
	DATA $0000,$0000,$0000,$0000,$0000,$0034,$0048,$0045,$0000,$0022,$0045,$0041,$0053,$0054,$0000,$0000,$0000,$0000,$0000,$0000
levelHeader30:
	DATA $0007,$000a
	DATA $0001,$0001
levelRowBits30:
	DATA $5a00,$aec0,$2a00,$6dc0,$2a80,$6b80,$2e80

REM Level 31 "Speaking Frankly 1"
levelName31:
	DATA $0000,$0033,$0050,$0045,$0041,$004b,$0049,$004e,$0047,$0000,$0026,$0052,$0041,$004e,$004b,$004c,$0059,$0000,$0011,$0000
levelHeader31:
	DATA $0004,$0007
	DATA $0004,$0001
levelRowBits31:
	DATA $f800,$b400,$9a00,$0000

REM Level 32 "Speaking Frankly 2"
levelName32:
	DATA $0000,$0033,$0050,$0045,$0041,$004b,$0049,$004e,$0047,$0000,$0026,$0052,$0041,$004e,$004b,$004c,$0059,$0000,$0012,$0000
levelHeader32:
	DATA $0006,$000a
	DATA $0006,$0001
levelRowBits32:
	DATA $ff00,$f680,$f340,$b680,$9ac0,$0000

REM Level 33 "Speaking Frankly 3"
levelName33:
	DATA $0000,$0033,$0050,$0045,$0041,$004b,$0049,$004e,$0047,$0000,$0026,$0052,$0041,$004e,$004b,$004c,$0059,$0000,$0013,$0000
levelHeader33:
	DATA $0008,$000d
	DATA $0008,$0001
levelRowBits33:
	DATA $ffe0,$fed0,$fe68,$f6d0,$f358,$b6d0,$9ad8,$0000

REM Level 34 "Speaking Frankly 4"
levelName34:
	DATA $0000,$0033,$0050,$0045,$0041,$004b,$0049,$004e,$0047,$0000,$0026,$0052,$0041,$004e,$004b,$004c,$0059,$0000,$0014,$0000
levelHeader34:
	DATA $000a,$0010
	DATA $000a,$0001
levelRowBits34:
	DATA $fffc,$ffda,$ffcd,$feda,$fe6b,$f6da,$f35b,$b6da,$9adb,$0000

REM Level 35 "Speaking Frankly 5"
levelName35:
	DATA $0000,$0033,$0050,$0045,$0041,$004b,$0049,$004e,$0047,$0000,$0026,$0052,$0041,$004e,$004b,$004c,$0059,$0000,$0015,$0000
levelHeader35:
	DATA $000c,$0013
	DATA $000c,$0001
levelRowBits35:
	DATA $ffff,$fffb,$fff9,$ffdb,$ffcd,$fedb,$fe6b,$f6db,$f35b,$b6db,$9adb,$0000

REM
REM Bitmap Data
REM ----------
REM 1-bit bitmap data (including unused images) below.
REM Note that the player sprite uses multiple sprites on top of each other.
REM


' GRAM cards (there are up to 64 of them)
bitmapData:

' standing hair
BITMAP "..####.." 
BITMAP ".##..##."
BITMAP ".#....#."
BITMAP ".#....#."
BITMAP "#......#"
BITMAP "........"
BITMAP "........"
BITMAP "........"

' standing skin
BITMAP "........"
BITMAP "...##..."
BITMAP "..####.."
BITMAP "..####.."
BITMAP "...##..."
BITMAP "........"
BITMAP ".#....#."
BITMAP "#......#"

' standing shirt
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "..####.."
BITMAP "...##..."
BITMAP "...##..."

' standing pants
BITMAP "...##..."
BITMAP "...##..."
BITMAP "...##..."
BITMAP "..#..#.."
BITMAP "..#..#.."
BITMAP "..#..#.."
BITMAP "..#..#.."
BITMAP ".##..##."


' facing right hair
BITMAP ".####..." 
BITMAP "##..#..."
BITMAP "#......."
BITMAP "#......."
BITMAP "#......."
BITMAP "........"
BITMAP "........"
BITMAP "........"

' facing right skin
BITMAP "........"
BITMAP "..##...."
BITMAP ".####..."
BITMAP ".####..."
BITMAP "..##...."
BITMAP "........"
BITMAP "..#....."
BITMAP "..#....."

' facing right shirt
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "..##...."
BITMAP "...#...."
BITMAP "...#...."

' facing right pants
BITMAP "..##...."
BITMAP "..##...."
BITMAP "..##...."
BITMAP ".#..#..."
BITMAP ".#..#..."
BITMAP ".#..#..."
BITMAP ".#..#..."
BITMAP ".##.##.."

' pushing right hair
BITMAP "..####.." 
BITMAP ".##....."
BITMAP ".#......"
BITMAP "#......."
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "........"

' pushing right skin
BITMAP "........"
BITMAP "...##..."
BITMAP "..####.."
BITMAP "..####.."
BITMAP "...#...."
BITMAP "...#.###"
BITMAP "........"
BITMAP "........"

' pushing right shirt
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "........"
BITMAP "....#..."
BITMAP "...##..."
BITMAP "...##..."

' pushing right pants
BITMAP "...##..."
BITMAP "...##..."
BITMAP "...##..."
BITMAP "..#..#.."
BITMAP "..#..#.."
BITMAP ".#..#..."
BITMAP "#..#...."
BITMAP "##.##..."
